/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.codegen;

import java.util.HashMap;
import java.util.List;

import mc.DSLRoot;
import mc.ast.ASTNode;
import mc.codegen.TemplateCalculator;
import mc.codegen.TemplateOperator;
import mc.umlp.arcd._ast.ASTArcComponent;
import mc.umlp.arcd._ast.ASTArcComponentImplementation;
import mc.umlp.arcd._ast.ASTArcElement;
import mc.umlp.arcd.ets.entries.ComponentEntry;
import mc.umlp.arcd.ets.entries.PortEntry;
import mc.umlp.sc._ast.ASTMCCompilationUnit;
import mc.umlp.sc._ast.ASTSCDefinition;
import mc.umlp.sc._ast.ASTSCTransition;
import mc.umlp.sc._ast.SCNodeFactory;
import mc.umlp.sc.codegen.Helper;

/**
 * Validates, if the passed {@link ASTArcElement} corresponds the a statechart
 * implementation of the component's behavior.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3032 $, $Date: 2014-10-11 18:37:43 +0000 (Sat, 11 Oct 2014) $
 * @since 2.5.0
 */
public class IsSCImplCalculator extends TemplateCalculator<ASTArcElement> {
  
  /**
   * @see mc.codegen.TemplateCalculator#calc(mc.ast.ASTNode, mc.codegen.TemplateOperator)
   */
  @Override
  public boolean calc(ASTArcElement ast, TemplateOperator op) {
    if (ast instanceof ASTArcComponentImplementation && ((ASTArcComponentImplementation) ast).getKind().equals("sc")) {
      ASTArcComponent comp = (ASTArcComponent) ast.get_Parent().get_Parent().get_Parent();
      ComponentEntry compEntry = (ComponentEntry) getSymboltable().resolve(comp.getName(), ComponentEntry.KIND, ast);
      op.setValue("compEntry", compEntry);
      
      List<PortEntry> incomingPorts = compEntry.getAllIncomingPorts(getSymboltable().getModelloader(), getSymboltable().getDeserializers());
      StringBuilder portAndType = new StringBuilder();
      String sep = "";
      for (PortEntry p : incomingPorts) {
        portAndType.append(sep);
        portAndType.append(p.getTypeReference().print());
        portAndType.append(" ");
        portAndType.append(p.getName());
        sep = ", ";
      }
      op.setValue("portAndType", portAndType);
      ASTSCDefinition scDef = (ASTSCDefinition) ((ASTArcComponentImplementation) ast).getBehaviorEmbedding().deepClone();
      de.montiarc.extension.examples.syntax._ast.ASTMCCompilationUnit maComp;
      ASTNode parent = comp;
      while (comp.getMainParent() != null ) {
        parent = comp.getMainParent();
      }
      maComp = (de.montiarc.extension.examples.syntax._ast.ASTMCCompilationUnit) parent.get_Parent();
      
      
      setRoots(scDef, maComp.get_root());
      
      ASTMCCompilationUnit scComp = SCNodeFactory.createASTMCCompilationUnit(maComp.getImportStatements().deepClone(), maComp.getPackage(), scDef, "");
      scComp.set_root(maComp.get_root());
      
      HashMap<String, List<ASTSCTransition>> mToTrans = Helper.transWithSameTrigger(scDef.getSCTransitions());
      HashMap<String, List<ASTSCTransition>> rtcMap = new HashMap<String, List<ASTSCTransition>>();
      
      if (mToTrans.keySet().size() > 1) {
        getSymboltable().getErrorDelegator().addError("0xU0106 Mmay only have spont. trans!");
      }
      else {
        rtcMap = Helper.transWithSameSourceState(mToTrans.get("rtc"));
        op.setValue("rtcMap", rtcMap);        
      }
      return true;
    }
    return false;
  }

  /**
   * TODO: Write me!
   * @param scDef
   * @param get_root
   */
  private void setRoots(ASTNode scDef, DSLRoot<?> root) {
    scDef.set_root(root);
    for (ASTNode child : scDef.get_Children()) {
      setRoots(child, root);
    }
  }
  
}
