/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.processing;

import static mc.umlp.arc.MontiArcConstants.MONTI_ARC_ROOT_NAME;
import static mc.umlp.arcd.ToolConstants.ARG_ANALYSIS;
import static mc.umlp.arcd.ToolConstants.ARG_OUT;
import static mc.umlp.arcd.ToolConstants.ARG_SYMTABDIR;
import static mc.umlp.arcd.ToolConstants.ARG_SYNTHESIS;
import static mc.umlp.arcd.ToolConstants.WF_PARSE;
import interfaces2.language.ILanguage;
import interfaces2.language.ModelingLanguage;
import mc.DSLWorkflow;
import mc.umlp.arc.MontiArcDefaultModule;
import mc.umlp.arc.MontiArcLanguage;
import mc.umlp.arc.MontiArcTool;

import com.google.inject.Inject;
import com.google.inject.Provides;

import de.montiarc.extension.examples.helper.ToolCfgConstants;

//->AddWorkflow.java<@ignoreComments
/**
 * Demonstrates how to add a new {@link DSLWorkflow} to a {@link MontiArcTool}.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3055 $, $Date: 2014-12-09 17:34:07 +0000 (Tue, 09 Dec 2014) $
 * @since 2.5.0
 */
public class AddWorkflow implements ToolCfgConstants {
  
  
  //->AddWorkflow.ProcExtModule.java<
  /** Name of the 'count ports' execution unit.*/
  public static final String COUNT_PORTS = "countPorts";
  
  public static class ProcExtModule extends MontiArcDefaultModule {
    @Override
    protected void bindMontiArcModelingLanguage() {
      // left empty...we simply override default binding.
    }
    @Inject @Provides
    ModelingLanguage getExtendedMontiArcLanguage(ILanguage component) {
      MontiArcLanguage l = new MontiArcLanguage(component);
      l.addExecutionUnit(COUNT_PORTS, PortCountWorkflow.create());
      return l;
    }
  }
  //->AddWorkflow.ProcExtModule.java<
  
  public static void main(String[] args) {
    //->AddWorkflow.main.java<
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/processing",
        ARG_OUT, OUT_DIR,
        ARG_SYMTABDIR, SYMTAB_DIR,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_PARSE,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, COUNT_PORTS
    };
    // create tool with custom dependency injection module
    MontiArcTool t = new MontiArcTool(toolArgs, new ProcExtModule());
    // execute tool
    t.run();
    //->AddWorkflow.main.java<
  }
}
// ->AddWorkflow.java<
