/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.processing;

import interfaces2.helper.SymbolTableInterface;

import java.util.List;

import mc.umlp.arc.ets.transform.MontiArcTrafoConfigurationFactory;
import mc.umlp.arcd.ToolConstants;
import mc.umlp.arcd.ets.transform.ITrafoConfiguration;
import mc.umlp.arcd.ets.transform.transformators.ITransformator;

import com.google.common.base.Optional;
import com.google.common.collect.Lists;

/**
 * Custom factory that adds new transformations.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3030 $, $Date: 2014-10-09 17:34:28 +0000 (Thu, 09 Oct 2014) $
 * @since 2.5.0
 */
//->CustomTrafoConfigFactory.java<
public class CustomTrafoConfigFactory extends MontiArcTrafoConfigurationFactory {
  
  @Override
  public Optional<ITrafoConfiguration> create(final SymbolTableInterface symtab, String workflow) {
    // get MontiArc transformations from super class for the given workflow
    final Optional<ITrafoConfiguration> maTrafos = super.create(symtab, workflow);
    ITrafoConfiguration cfg = null;
    // add a transformation that is executed before context condition checks
    if (workflow.equals(ToolConstants.WF_PRE_CHECK_TRAFO)) {
      cfg = new ITrafoConfiguration() {
        private final List<ITransformator> trafos = Lists.newArrayList();
        @Override
        public List<ITransformator> getTrafos() {
          if (trafos.isEmpty()) {
            // add custom transformation
            trafos.add(new ExpandSubcomponentTrafo(symtab));
            // add MontiArc transformations, if present
            if (maTrafos.isPresent()) {
              trafos.addAll(maTrafos.get().getTrafos());
            }
          }
          return trafos;
        }
      };
    }
    if (cfg != null) {
      return Optional.of(cfg);
    }
    else if (maTrafos.isPresent()) {
      return maTrafos;
    }
    // no transformation from this factory or a superclass
    else {
      return Optional.absent();
    }
  }
}
//->CustomTrafoConfigFactory.java<
