/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.processing;

import mc.DSLRoot;
import mc.DSLWorkflow;
import mc.helper.GeneratedFile;
import mc.helper.GeneratedFileHelper;
import mc.umlp.arc._ast.ASTMCCompilationUnit;
import mc.umlp.arc._tool.MontiArcRoot;
import mc.umlp.arcd._ast.ASTArcComponent;
import mc.umlp.arcd._ast.ASTArcElement;
import mc.umlp.arcd._ast.ASTArcInterface;

//->PortCountWorkflow.java<@ignoreComments
/**
 * A simple workflow that counts ports to demonstrate how to integrate new
 * workflows into the MontiArc tool.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3048 $, $Date: 2014-10-06 16:48:20 +0200 (Mo, 06 Okt
 * 2014) $
 * 
 * @since 2.5.0
 */
public class PortCountWorkflow<T extends DSLRoot<? extends ASTMCCompilationUnit>> extends DSLWorkflow<T> {
  
  /**
   * 
   * @return a new {@link PortCountWorkflow} configured for the 
   * {@link MontiArcRoot}.
   */
  public static PortCountWorkflow<MontiArcRoot> create() {
      return new PortCountWorkflow<MontiArcRoot>(MontiArcRoot.class);
  }
  /**
   * Constructor for de.montiarc.extension.examples.processing.PortCountWorkflow
   * 
   * @param responsibleClass
   */
  public PortCountWorkflow(Class<T> rootClass) {
    super(rootClass);
  }
  
  /**
   * @see mc.DSLWorkflow#run(mc.DSLRoot)
   */
  @Override
  public void run(T dslroot) {
    ASTArcComponent component = dslroot.getAst().getType();
    int ports = countPorts(component);
    
    StringBuilder sb = new StringBuilder();
    sb.append("Component ");
    sb.append(component.getName());
    sb.append(" has ");
    sb.append(ports);
    sb.append(" ports.");
    GeneratedFile result = GeneratedFileHelper.createDeferredFileFromQualifiedName(
        dslroot.getName(), "stats");
    result.setContent(sb);
    dslroot.addFile(result);
  }
  
  /**
   * Counts all ports of the given component. Ports of inner component
   * definitions are also counted.
   * 
   * @param component the ports of this component are counted.
   * @return amount of ports from the given component and its contained inner
   * component definitions.
   */
  private int countPorts(ASTArcComponent component) {
    int result = 0;
    for (ASTArcElement element : component.getBody().getArcElement()) {
      if (element instanceof ASTArcInterface) {
        result += ((ASTArcInterface) element).getPorts().size();
      }
      else if (element instanceof ASTArcComponent) {
        result += countPorts(((ASTArcComponent) element));
      }
    }
    return result;
  }
}
//->PortCountWorkflow.java<
