/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.codegen;

import static mc.umlp.arc.MontiArcConstants.MONTI_ARC_ROOT_NAME;
import static mc.umlp.arcd.ToolConstants.ARG_ANALYSIS;
import static mc.umlp.arcd.ToolConstants.ARG_GENERATOR;
import static mc.umlp.arcd.ToolConstants.ARG_MODELPATH;
import static mc.umlp.arcd.ToolConstants.ARG_OUT;
import static mc.umlp.arcd.ToolConstants.ARG_SYMTABDIR;
import static mc.umlp.arcd.ToolConstants.ARG_SYNTHESIS;
import static mc.umlp.arcd.ToolConstants.PARAM_ALL;
import static mc.umlp.arcd.ToolConstants.ROOT_NAME_JAVA;
import static mc.umlp.arcd.ToolConstants.WF_CREATE_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_INIT_CHECK;
import static mc.umlp.arcd.ToolConstants.WF_INIT_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_PARSE;
import static mc.umlp.arcd.ToolConstants.WF_PRE_CHECK_TRAFO;
import static mc.umlp.arcd.ToolConstants.WF_RUN_CHECK;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;

import mc.umlp.arc.MontiArcGeneratorConstants;
import mc.umlp.arc.MontiArcGeneratorTool;

import org.apache.commons.io.FileUtils;
import org.junit.Before;
import org.junit.Test;

import sim.IScheduler;
import sim.error.ISimulationErrorHandler;
import sim.error.SimpleErrorHandler;
import sim.generic.IStream;
import sim.generic.Tick;
import sim.port.IInPort;
import sim.port.IPort;
import sim.port.TestPort;
import sim.sched.SchedulerFactory;

import com.google.inject.AbstractModule;

import de.montiarc.extension.examples.helper.ToolCfgConstants;
import de.montiarc.extension.examples.syntax._ast.ASTMCCompilationUnit;

/**
 * Tests for the MontiArc with embedded SC code generator.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3114 $, $Date: 2015-02-05 16:20:14 +0000 (Thu, 05 Feb 2015) $
 * @since 2.5.0
 */
public class TestCodegen implements ToolCfgConstants {
  
  private final String startNode = ASTMCCompilationUnit.class.getName();
  
  private final String targetDir = OUT_DIR;
  
  @Before
  public void setup() {
    // generate
    try {
      FileUtils.deleteDirectory(new File(targetDir));
    }
    catch (IOException e) {
      // ignore
    }
    
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/syntax/MontiArcWithSCComp.arc",
        ARG_OUT, targetDir,
        ARG_SYMTABDIR, SYMTAB_DIR,
        ARG_MODELPATH, "src/main/resources",
        ARG_ANALYSIS, PARAM_ALL, WF_PARSE,
        ARG_ANALYSIS, ROOT_NAME_JAVA, "setname",
        ARG_ANALYSIS, ROOT_NAME_JAVA, "addImports",
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_INIT_SYMTAB,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_CREATE_SYMTAB,
        ARG_SYNTHESIS, PARAM_ALL, WF_INIT_CHECK,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, WF_PRE_CHECK_TRAFO,
        ARG_SYNTHESIS, PARAM_ALL, WF_RUN_CHECK,
        ARG_GENERATOR, "de.montiarc.extension.examples.codegen.ComponentWithSCMain", startNode,
        ARG_GENERATOR, MontiArcGeneratorConstants.INTERFACE_GENERATOR, startNode,
        ARG_GENERATOR, "de.montiarc.extension.examples.codegen.ComponentWithSCFactoryMain",
        startNode
    };
    // create tool with custom dependency injection module
    AbstractModule mod = new SyntaxExtGenModule();
    MontiArcGeneratorTool tool = new MontiArcGeneratorTool(toolArgs, mod);
    // execute tool
    assertTrue(tool.run());
    File comp = new File(targetDir + "/syntax/gen/MontiArcWithSCComp.java");
    File interf = new File(targetDir + "/syntax/gen/interfaces/IMontiArcWithSCComp.java");
    File fact = new File(targetDir + "/syntax/gen/factories/MontiArcWithSCCompFactory.java");
    
    assertTrue(comp.exists());
    assertTrue(interf.exists());
    assertTrue(fact.exists());
    
    // compile
    // Compile source file.
    JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
    compiler.run(null, null, null, comp.getPath(), interf.getPath(), fact.getPath());
    
  }
  
  private Object load() {
    File root = new File(targetDir);
    // Load and instantiate compiled class.
    URLClassLoader classLoader;
    try {
      classLoader = URLClassLoader.newInstance(new URL[] { root.toURI().toURL() });
      Class<?> cls = Class.forName("syntax.gen.MontiArcWithSCComp", true, classLoader);
      Object instance = cls.newInstance();
      assertNotNull(instance);
      return instance;
    }
    catch (Exception e) {
      e.printStackTrace();
      fail(e.getMessage());
      return null;
    }
  }
  
  @Test
  public void testExecute() {
    Object component = load();
    TestPort<Boolean> finishedPort = setup(component);
    IInPort<Boolean> pingPort = getPingPort(component);
    final Tick<Boolean> tick = Tick.<Boolean> get();
    int time = 10;
    for (int i = 0; i < time; i++) {
      pingPort.accept(tick);
      if (i % 2 == 0) {
        pingPort.accept(true);
      }
      else {
        pingPort.accept(false);
      }
    }
    System.out.println("Produced Stream: " + finishedPort.getStream());
    IStream<Boolean> stream = finishedPort.getStream();
    assertEquals(time, stream.getCurrentTime());
    
    for (int i = 3; i < time; i++) {
      if (i % 2 == 0) {
        assertEquals(1, stream.getTimeInterval(i).size());
        assertTrue(stream.getTimeInterval(i).get(0));
      }
      else {
        assertTrue(stream.getTimeInterval(i).isEmpty());
      }
    }
  }
  
  @SuppressWarnings("unchecked")
  private IInPort<Boolean> getPingPort(Object component) {
    try {
      IInPort<Boolean> result;
      Method getPing = component.getClass().getMethod("getPing");
      result = (IInPort<Boolean>) getPing.invoke(component);
      return result;
    }
    catch (Exception e) {
      e.printStackTrace();
      fail(e.getMessage());
    }
    
    return null;
  }
  
  private TestPort<Boolean> setup(Object component) {
    try {
      Method setup = component.getClass().getMethod("setup", IScheduler.class,
          ISimulationErrorHandler.class);
      setup.invoke(component, SchedulerFactory.createDefaultScheduler(), new SimpleErrorHandler());
      TestPort<Boolean> t = new TestPort<Boolean>();
      
      Method setFinished = component.getClass().getMethod("setFinished", IPort.class);
      setFinished.invoke(component, t);
      return t;
    }
    catch (Exception e) {
      e.printStackTrace();
      fail(e.getMessage());
      return null;
    }
    
  }
  
}
