/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.processing;

import static mc.umlp.arc.MontiArcConstants.MONTI_ARC_ROOT_NAME;
import static mc.umlp.arcd.ToolConstants.ARG_ANALYSIS;
import static mc.umlp.arcd.ToolConstants.ARG_MODELPATH;
import static mc.umlp.arcd.ToolConstants.ARG_OUT;
import static mc.umlp.arcd.ToolConstants.ARG_SYNTHESIS;
import static mc.umlp.arcd.ToolConstants.PARAM_ALL;
import static mc.umlp.arcd.ToolConstants.ROOT_NAME_JAVA;
import static mc.umlp.arcd.ToolConstants.WF_CREATE_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_INIT_CHECK;
import static mc.umlp.arcd.ToolConstants.WF_INIT_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_PARSE;
import static mc.umlp.arcd.ToolConstants.WF_PRETTY_PRINT;
import static mc.umlp.arcd.ToolConstants.WF_PRE_CHECK_TRAFO;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;

import mc.umlp.arc.MontiArcTool;
import mc.umlp.arc._tool.MontiArcRoot;
import mc.umlp.arcd._ast.ASTArcComponent;

import org.apache.commons.io.FileUtils;
import org.junit.Before;
import org.junit.Test;

import de.montiarc.extension.examples.helper.ToolCfgConstants;

/**
 * Tests adding custom trafos.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3032 $, $Date: 2014-10-11 18:37:43 +0000 (Sat, 11 Oct 2014) $
 * @since 2.5.0
 */
public class AddTrafoTest implements ToolCfgConstants {
  
  @Before
  public void setUp() {
    try {
      FileUtils.deleteDirectory(new File(OUT_DIR));
    }
    catch (IOException e) {
      // ignore
    }
  }
  
  @Test
  public void testIfAddedWorkflowIsExecuted() {
    final String file = "src/main/resources/processing/Expand.arc";
    String[] toolArgs = new String[] {
        file,
        ARG_MODELPATH, "src/main/resources/",
        ARG_OUT, OUT_DIR,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_PARSE,
        ARG_ANALYSIS, ROOT_NAME_JAVA, "setname",
        ARG_ANALYSIS, ROOT_NAME_JAVA, "addImports",
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_INIT_SYMTAB,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_CREATE_SYMTAB,
        ARG_SYNTHESIS, PARAM_ALL, WF_INIT_CHECK,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, WF_PRE_CHECK_TRAFO,
        ARG_SYNTHESIS, PARAM_ALL, WF_PRETTY_PRINT,
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new AddTrafo.AddTrafoModule());
    // execute tool
    assertTrue(tool.run());
    
    File one = new File(OUT_DIR + "/Expand.arc");
    assertTrue(one.exists());
    
    MontiArcRoot root = (MontiArcRoot) tool.getRootByFileName(file);
    assertNotNull(root);
    
    ASTArcComponent transformed = root.getAst().getType();
    assertTrue(transformed.getInnerComponents().size() == 1);
    assertEquals(transformed.getInnerComponents().get(0).getName(), "Processing");
    
  }
  
}
