/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.processing;

import static mc.umlp.arc.MontiArcConstants.MONTI_ARC_ROOT_NAME;
import static mc.umlp.arcd.ToolConstants.ARG_ANALYSIS;
import static mc.umlp.arcd.ToolConstants.ARG_OUT;
import static mc.umlp.arcd.ToolConstants.ARG_SYNTHESIS;
import static mc.umlp.arcd.ToolConstants.WF_PARSE;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;

import mc.umlp.arc.MontiArcTool;

import org.apache.commons.io.FileUtils;
import org.junit.Before;
import org.junit.Test;

import de.montiarc.extension.examples.helper.ToolCfgConstants;
import de.montiarc.extension.examples.processing.AddWorkflow.ProcExtModule;

/**
 * Tests adding new workflows.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3055 $, $Date: 2014-10-06 18:20:17 +0200 (Mo, 06 Okt
 * 2014) $
 * @since 2.5.0
 */
public class AddWorkflowTest implements ToolCfgConstants {
  
  @Before
  public void setUp() {
    try {
      FileUtils.deleteDirectory(new File(OUT_DIR));
    }
    catch (IOException e) {
      // ignore
    }
  }
  
  @Test
  public void testIfAddedWorkflowIsExecuted() {
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/processing",
        ARG_OUT, OUT_DIR,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_PARSE,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, AddWorkflow.COUNT_PORTS
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new ProcExtModule());
    // execute tool
    tool.run();
    
    File one = new File(OUT_DIR + "/processing/Processing.stats");
    File two = new File(OUT_DIR + "/processing/ProcessingWithInner.stats");
    assertTrue(one.exists());
    assertTrue(two.exists());
    
    try {
      assertTrue(FileUtils.readFileToString(one).contains("2"));
      assertTrue(FileUtils.readFileToString(two).contains("5"));
    }
    catch (IOException e) {
      fail(e.getMessage());
    }
  }
}
