/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.symtab;

import static mc.umlp.arc.MontiArcConstants.MONTI_ARC_ROOT_NAME;
import static mc.umlp.arcd.ToolConstants.ARG_ANALYSIS;
import static mc.umlp.arcd.ToolConstants.ARG_MODELPATH;
import static mc.umlp.arcd.ToolConstants.ARG_SYMTABDIR;
import static mc.umlp.arcd.ToolConstants.ARG_SYNTHESIS;
import static mc.umlp.arcd.ToolConstants.PARAM_ALL;
import static mc.umlp.arcd.ToolConstants.ROOT_NAME_JAVA;
import static mc.umlp.arcd.ToolConstants.WF_CREATE_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_INIT_CHECK;
import static mc.umlp.arcd.ToolConstants.WF_INIT_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_PARSE;
import static mc.umlp.arcd.ToolConstants.WF_PRE_CHECK_TRAFO;
import static mc.umlp.arcd.ToolConstants.WF_RUN_CHECK;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import interfaces2.ISTEntry;
import interfaces2.helper.SymbolTableInterface;
import interfaces2.loaders.InterfaceLoader;

import java.io.File;
import java.io.IOException;
import java.util.Set;

import mc.umlp.arc.MontiArcConstants;
import mc.umlp.arc.MontiArcTool;
import mc.umlp.arcd.ets.entries.AComponentImplementationEntry;
import mc.umlp.arcd.ets.entries.ComponentEntry;

import org.apache.commons.io.FileUtils;
import org.junit.Before;
import org.junit.Test;

import com.google.common.base.Optional;

import de.montiarc.extension.examples.helper.ToolCfgConstants;
import de.montiarc.extension.examples.syntax.SyntaxExtModule;
import de.montiarc.extension.examples.syntax._tool.ExtendedMontiArcRoot;
import de.montiarc.extension.examples.syntax.ets.ArcSCEntry;
import de.montiarc.extension.examples.syntax.ets.ArcSCImplementationEntry;

/**
 * Tests for symbol table extensions.
 *
 * @author  (last commit) $Author: ahaber $
 * @version $Revision: 3032 $,
 *          $Date: 2014-10-11 18:37:43 +0000 (Sat, 11 Oct 2014) $
 * @since   2.5.0
 */
public class MontiArcSymtabExtensionTest implements ToolCfgConstants{
  
  @Before
  public void setUp() {
    try {
      FileUtils.deleteDirectory(new File(SYMTAB_DIR));
    }
    catch (IOException e) {
      // ignore
    }
  }
  
  @Test
  public void testSymtabOfEmbedding() {
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/syntax/MontiArcWithSCComp.arc",
        ARG_MODELPATH, "src/main/resources",
        ARG_SYMTABDIR, SYMTAB_DIR,
        ARG_ANALYSIS, PARAM_ALL, WF_PARSE,
        ARG_ANALYSIS, ROOT_NAME_JAVA, "setname",
        ARG_ANALYSIS, ROOT_NAME_JAVA, "addImports",
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_INIT_SYMTAB,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_CREATE_SYMTAB,
        ARG_SYNTHESIS, PARAM_ALL, WF_INIT_CHECK,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, WF_PRE_CHECK_TRAFO,
        ARG_SYNTHESIS, PARAM_ALL, WF_RUN_CHECK,
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new SyntaxExtModule());
    // execute tool
    assertTrue(tool.run());
    ExtendedMontiArcRoot root = (ExtendedMontiArcRoot) tool
        .getRootByFileName("src/main/resources/syntax/MontiArcWithSCComp.arc");
    assertNotNull(root);
    SymbolTableInterface symtab = new SymbolTableInterface(root);
    ComponentEntry entry = (ComponentEntry) symtab.resolve("syntax.MontiArcWithSCComp", ComponentEntry.KIND, root.getAst().getType());
    Optional<AComponentImplementationEntry> impl = entry.getImplementation("PingPongImpl");
    assertTrue(impl.isPresent());
    ArcSCImplementationEntry scImpl = (ArcSCImplementationEntry) impl.get();
    ArcSCEntry scEntry = scImpl.getSc();
    assertEquals("PingPong", scEntry.getName());
    assertEquals(2, scEntry.getStates().size());
    assertTrue(scEntry.getStates().contains("Ping"));
    assertTrue(scEntry.getStates().contains("Pong"));
    
  }
  
  @Test
  public void testDeserializeModel() {
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/syntax/MontiArcWithSCComp.arc",
        ARG_MODELPATH, "src/main/resources",
        ARG_SYMTABDIR, SYMTAB_DIR,
        ARG_ANALYSIS, PARAM_ALL, WF_PARSE,
        ARG_ANALYSIS, ROOT_NAME_JAVA, "setname",
        ARG_ANALYSIS, ROOT_NAME_JAVA, "addImports",
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_INIT_SYMTAB,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_CREATE_SYMTAB,
        ARG_SYNTHESIS, PARAM_ALL, WF_INIT_CHECK,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, WF_PRE_CHECK_TRAFO,
        ARG_SYNTHESIS, PARAM_ALL, WF_RUN_CHECK,
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new SyntaxExtModule());
    // execute tool
    assertTrue(tool.run());
    ExtendedMontiArcRoot root = (ExtendedMontiArcRoot) tool
        .getRootByFileName("src/main/resources/syntax/MontiArcWithSCComp.arc");
    assertNotNull(root);
    SymbolTableInterface symtab = new SymbolTableInterface(root);
    
    Set<ISTEntry> protectedEntries = InterfaceLoader.loadExported(symtab.getModelloader(), "syntax.MontiArcWithSCComp", MontiArcConstants.ST_KIND_PROTECTED, symtab.getDeserializers(), ComponentEntry.KIND);
    assertEquals(1, protectedEntries.size());
    
    ComponentEntry protectedEntry = (ComponentEntry) protectedEntries.iterator().next();
    
    Optional<AComponentImplementationEntry> impl = protectedEntry.getImplementation("PingPongImpl");
    assertTrue(impl.isPresent());
    ArcSCImplementationEntry scImpl = (ArcSCImplementationEntry) impl.get();
    ArcSCEntry scEntry = scImpl.getSc();
    assertEquals("PingPong", scEntry.getName());
    assertEquals(2, scEntry.getStates().size());
    assertTrue(scEntry.getStates().contains("Ping"));
    assertTrue(scEntry.getStates().contains("Pong"));
    
  }
  
}
