/*
 * Copyright (c) 2014 RWTH Aachen. All rights reserved.
 *
 * http://www.se-rwth.de/
 */
package de.montiarc.extension.examples.syntax;

import static mc.umlp.arc.MontiArcConstants.MONTI_ARC_ROOT_NAME;
import static mc.umlp.arcd.ToolConstants.ARG_ANALYSIS;
import static mc.umlp.arcd.ToolConstants.ARG_MODELPATH;
import static mc.umlp.arcd.ToolConstants.ARG_SYNTHESIS;
import static mc.umlp.arcd.ToolConstants.PARAM_ALL;
import static mc.umlp.arcd.ToolConstants.ROOT_NAME_JAVA;
import static mc.umlp.arcd.ToolConstants.WF_CREATE_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_INIT_CHECK;
import static mc.umlp.arcd.ToolConstants.WF_INIT_SYMTAB;
import static mc.umlp.arcd.ToolConstants.WF_PARSE;
import static mc.umlp.arcd.ToolConstants.WF_PRE_CHECK_TRAFO;
import static mc.umlp.arcd.ToolConstants.WF_RUN_CHECK;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import mc.test.helpers.TestErrorHandler;
import mc.umlp.arc.MontiArcTool;
import mc.umlp.arcd._ast.ASTArcComponentImplementation;
import mc.umlp.sc._ast.ASTSCDefinition;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.montiarc.extension.examples.helper.ToolCfgConstants;
import de.montiarc.extension.examples.syntax._ast.ASTPropertySet;
import de.montiarc.extension.examples.syntax._ast.ASTSingleProperty;
import de.montiarc.extension.examples.syntax._tool.ExtendedMontiArcRoot;

/**
 * Acceptance test for the extended language.
 * 
 * @author (last commit) $Author: ahaber $
 * @version $Revision: 3032 $, $Date: 2014-10-09 11:04:36 +0200 (Do, 09 Okt
 * 2014) $
 * @since 2.5.0
 */
public class MontiArcSyntaxExtensionTests implements ToolCfgConstants{
  
  @Before
  @After
  public void resetErrorHandler() {
    TestErrorHandler.getInstance().clearErrorList();
  }
  
  @Test
  public void testParseInheritance() {
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/syntax/ExtendedMontiArcComponent.arc",
        ARG_MODELPATH, "src/main/resources",
        ARG_ANALYSIS, PARAM_ALL, WF_PARSE
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new SyntaxExtModule());
    // execute tool
    boolean success = tool.run();
    assertTrue(success);
    
    ExtendedMontiArcRoot root = (ExtendedMontiArcRoot) tool
        .getRootByFileName("src/main/resources/syntax/ExtendedMontiArcComponent.arc");
    assertNotNull(root);
    ASTSingleProperty single = (ASTSingleProperty) root.getAst().getType().getBody()
        .getArcElement().get(0);
    assertNotNull(single);
    assertEquals("delay", single.getProperty().getName());
    
    ASTPropertySet list = (ASTPropertySet) root.getAst().getType().getBody().getArcElement().get(1);
    assertNotNull(list);
    assertEquals(2, list.getProperties().size());
  }
  
  @Test
  public void testUseCoCosInExtendedModel() {
    TestErrorHandler eh = TestErrorHandler.getInstance();
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/syntax/ExtendedMontiArcComponentCoCoFail.arc",
        ARG_MODELPATH, "src/main/resources",
        ARG_ANALYSIS, PARAM_ALL, WF_PARSE,
        ARG_ANALYSIS, ROOT_NAME_JAVA, "setname",
        ARG_ANALYSIS, ROOT_NAME_JAVA, "addImports",
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_INIT_SYMTAB,
        ARG_ANALYSIS, MONTI_ARC_ROOT_NAME, WF_CREATE_SYMTAB,
        ARG_SYNTHESIS, PARAM_ALL, WF_INIT_CHECK,
        ARG_SYNTHESIS, MONTI_ARC_ROOT_NAME, WF_PRE_CHECK_TRAFO,
        ARG_SYNTHESIS, PARAM_ALL, WF_RUN_CHECK,
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new SyntaxExtModule());
    tool.addErrorHandler(eh);
    // execute tool
    boolean success = tool.run();
    assertTrue(success);
    assertTrue(eh.getWarningMessages().size() == 1);
  }
  
  @Test
  public void testParseEmbedding() {
    // configuration to parse and count ports.
    String[] toolArgs = new String[] {
        "src/main/resources/syntax/MontiArcWithSCComp.arc",
        ARG_MODELPATH, "src/main/resources",
        ARG_ANALYSIS, PARAM_ALL, WF_PARSE,
    };
    // create tool with custom dependency injection module
    MontiArcTool tool = new MontiArcTool(toolArgs, new SyntaxExtModule());
    // execute tool
    boolean success = tool.run();
    assertTrue(success);
    ExtendedMontiArcRoot root = (ExtendedMontiArcRoot) tool
        .getRootByFileName("src/main/resources/syntax/MontiArcWithSCComp.arc");
    assertNotNull(root);
    ASTArcComponentImplementation impl = (ASTArcComponentImplementation) root.getAst().getType()
        .getBody().getArcElement().get(2);
    assertNotNull(impl);
    assertTrue(impl.getBehaviorEmbedding() instanceof ASTSCDefinition);
    ASTSCDefinition def = (ASTSCDefinition) impl.getBehaviorEmbedding();
    assertEquals(2, def.getSCStates().size());
    assertEquals(2, def.getSCStates().size());
  }
  
}
